//+------------------------------------------------------------------+
//|                            CandleWicksDisplay_BodyFullLength.mq5 |
//|                             Copyright  2011-2016, Andriy Moraru |
//+------------------------------------------------------------------+
#property copyright "Copyright  2011-2016, Andriy Moraru"
#property link      "http://www.earnforex.com"
#property version   "1.02"

#property description "Displays candle's body and full length."
#property description "Alerts you when the candle's body or length reaches a certain length."
#property description "Your e-mail settings are set in Tools -> Options -> Email."
#property description "Also displays the candle's body and full length above and below the candles."

// The indicator uses only objects for display, but the line below is required for it to work.
#property indicator_chart_window

input int DisplayBodyLimit = 5; // DisplayBodyLimit - in standard pips
input int DisplayLengthLimit = 5; // DisplayLengthLimit - in standard pips
input color DisplayBullColor = clrLimeGreen;
input color DisplayBearColor = clrRed;
input int DisplayDistance = 0; // DisplayDistance - Distance from High or Low to the chart label.
input int BodyLimit = 10; // BodyLimit - in pips, for alerts
input int FullLimit = 10; // FullLimit - in pips, for alerts
input bool WaitForClose = true; // Wait for a candle to close before checking length
input bool EmailAlert = true;
input bool SoundAlert = false;
input bool VisualAlert = false;

// Time of the bar of the last alert
datetime AlertDone;

double Poin;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
void OnInit()
{
	Poin = _Point;
	//Checking for unconvetional Point digits number
   if ((_Point == 0.00001) || (_Point == 0.001)) Poin *= 10;
}

//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
{
   ObjectsDeleteAll(0, "CWD-", -1, OBJ_TEXT);
}

//+------------------------------------------------------------------+
//| CandleWicks                                                      |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &Time[],
                const double &Open[],
                const double &High[],
                const double &Low[],
                const double &Close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
{
   string name, length;
   bool DoAlert = false;
   int index = rates_total - 1;
   
   if (WaitForClose) index = rates_total - 2;
   
   int counted_bars = prev_calculated;
   if (counted_bars > 0) counted_bars--;
   int limit = counted_bars;
   if (rates_total - counted_bars > 500) limit = rates_total - 500;
     
   for (int i = rates_total - 1; i >= limit; i--)
   {
      if (MathAbs(Close[i] - Open[i]) >= DisplayBodyLimit * Poin) // Body length display
      {
         name = "CWD-B-" + TimeToString(Time[i], TIME_DATE|TIME_MINUTES);
         length = DoubleToString(MathRound(MathAbs((Close[i] - Open[i])) / Poin), 0);
         if (ObjectFind(0, name) != -1) ObjectDelete(0, name);
         ObjectCreate(0, name, OBJ_TEXT, 0, Time[i], High[i] + DisplayDistance * Poin);
			ObjectSetString(0, name, OBJPROP_TEXT, length);
   		ObjectSetInteger(0, name, OBJPROP_FONTSIZE, 10);
		   ObjectSetString(0, name, OBJPROP_FONT, "Verdana");
		   ObjectSetInteger(0, name, OBJPROP_ANCHOR, ANCHOR_LOWER);
		   if (Open[i] <= Close[i]) ObjectSetInteger(0, name, OBJPROP_COLOR, DisplayBullColor);
		   else ObjectSetInteger(0, name, OBJPROP_COLOR, DisplayBearColor);
      }
      if (High[i] - Low[i] >= DisplayLengthLimit * Poin) // Full length display
      {
         name = "CWD-F-" + TimeToString(Time[i], TIME_DATE|TIME_MINUTES);
         length = DoubleToString(MathRound((High[i] - Low[i]) / Poin), 0);
         if (ObjectFind(0, name) != -1) ObjectDelete(0, name);
         ObjectCreate(0, name, OBJ_TEXT, 0, Time[i], Low[i]);
			ObjectSetString(0, name, OBJPROP_TEXT, length);
   		ObjectSetInteger(0, name, OBJPROP_FONTSIZE, 10);
		   ObjectSetString(0, name, OBJPROP_FONT, "Verdana");
		   ObjectSetInteger(0, name, OBJPROP_ANCHOR, ANCHOR_UPPER);
		   if (Open[i] <= Close[i]) ObjectSetInteger(0, name, OBJPROP_COLOR, DisplayBullColor);
		   else ObjectSetInteger(0, name, OBJPROP_COLOR, DisplayBearColor);
      }
   }

   if (AlertDone == Time[index]) return(0); // Already sent an alert for this candle.
   
   if (MathAbs(Close[index] - Open[index]) >= BodyLimit * Poin) DoAlert = true;
   if (High[index] - Low[index] >= BodyLimit * Poin) DoAlert = true;
   
   if (DoAlert)
   {
      if (VisualAlert) Alert(Symbol() + " " + TimeToString(TimeCurrent()) + " - candle length limit reached!");
      if (SoundAlert) PlaySound("alert.wav");
      if (EmailAlert) SendMail("CandleWick Alert", Symbol() + " " + TimeToString(TimeCurrent()) + " - candle length limit reached!");
      AlertDone = Time[index];
   }
      
   return(rates_total);
}
//+------------------------------------------------------------------+